#!/usr/local/bin/perl -w

# compoundSval2.pl version 0.1
# Program to take a Senseval lexical sample file and a file containing
# compound words, and identify the compounds in the senseval lexical
# file.
#
# Copyright (C) 2001-2002
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Satanjeev Banerjee, University of Minnesota, Duluth
# bane0025@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#-----------------------------------------------------------------------------
#                              Start of program
#-----------------------------------------------------------------------------

# we have to use commandline options, so use the necessary package!
use Getopt::Long;

# first check if no commandline options have been provided... in which case
# print out the usage notes!
if ( $#ARGV == -1 )
{
    &minimalUsageNotes();
    exit;
}

# now get the options!
GetOptions("version", "help");

# if help has been requested, print out help!
if ( defined $opt_help )
{
    $opt_help = $opt_help;
    &showHelp();
    exit;
}

# if version has been requested, show version!
if ( defined $opt_version )
{
    $opt_version = $opt_version;
    &showVersion();
    exit;
}

# get the wordsfile
$wordsFile = shift;
if (!defined ($wordsFile))
{
    print "Words file required!\n";
    exit;
}

# get the lex file
$lexFile = shift;
if (!defined ($lexFile))
{
    print "Lex file required!\n";
    exit;
}

# open and hashify the words file
open (WORDS, "$wordsFile") || die ("Couldnt open $wordsFile\n");
while (<WORDS>)
{
    chomp;
    $words{$_} = 1;
}
close WORDS;

# now open and convert the lex file
open (LEX, "$lexFile") || die ("Couldnt open $lexFile\n");

$insideContext = 0; 
while (<LEX>)
{
    if (!$insideContext)
    {
	if (/<context>/) { $insideContext = 1; }
	else { print; }
    }

    # we allow for the <context> tag to have contextual text on the same line!
    if ($insideContext) 
    {
	$insideContext = 0 if (/<\/context>/);
	print compoundify($_);
    }
}

sub compoundify
{
    my $string = shift;
    my $prefix = ""; # the part upto and including <context>
    my $suffix = ""; # the part include </context> and till the end of the string

    if ($string =~ s/(.*<context>)//) { $prefix = $1; }
    if ($string =~ s/(<\/context>.*)//) { $suffix = $1; }

    # now convert whats left in the string
    my @wordsArray = ();
    
    # remove the <head> </head> tags if present, and remember their "position"
    # ASSUMPTION: <head> and </head> will either both occur or neither occur.
    my $headPosition = -1; # -1 for not present

    my $left = "";
    my $rest = "";
    if ($string =~ /(.*)<head[^>]*>(.*)<\/head>(.*)/)
    {
	$left = $1;
	$rest = $2 . " " . $3;
	$headPosition = 0; # at LEAST 0!
    }
    else { $left = $string; }

    # get all the words into an array
    $left =~ s/<[^>]*>//g;
    $left =~ s/&(\w*;)+//g;
    while ($left =~ /(\w+)/g)
    {
	push @wordsArray, $1;
    }

    if ($headPosition == 0)
    {
	$headPosition = $#wordsArray + 1;
	
	$rest =~ s/<[^>]*>//g;
	$rest =~ s/&(\w*;)+//g;
	while ($rest =~ /(\w+)/g)
	{
	    push @wordsArray, $1;
	}
    }
	
    # now compoundify, GREEDILY!!
    my $firstPointer = 0;
    $string = "";

    while ($firstPointer <= $#wordsArray)
    {
	my $secondPointer = $#wordsArray;
	my $done = 0;
	while ($secondPointer > $firstPointer && !$done)
	{
	    my $temp = join ("_", @wordsArray[$firstPointer..$secondPointer]);

	    if (exists $words{$temp})
	    {
		if ($headPosition > -1 && $firstPointer <= $headPosition && $secondPointer >= $headPosition) 
		{ 
		    $string .= "<head>$temp</head> "; 
		}
		else 
		{ 
		    $string .= "$temp "; 
		}
		$done = 1;
	    }
	    else { $secondPointer--; }
	}

	if (!$done) 
	{ 
	    if ($firstPointer == $headPosition) 
	    { 
		$string .= "<head>$wordsArray[$firstPointer]</head> "; 
	    }
	    else 
	    { 
		$string .= "$wordsArray[$firstPointer] "; 
	    }
	}	    
	
	$firstPointer = $secondPointer + 1;
    }
    
    return ($prefix . $string . $suffix . "\n");
}



# thats it!

# function to output a minimal usage note when the user has not provided any
# commandline options
sub minimalUsageNotes
{
    print "Usage: compoundSval2.pl [OPTIONS] WORDSFILE LEXFILE\n";
    askHelp();
}

# function to output help messages for this program
sub showHelp
{
    print "Usage: compound.pl [OPTIONS] WORDSFILE LEXFILE\n\n";

    print "Finds all compound words in LEXFILE according to the list of such words\n";
    print "in WORDSFILE, and connects the components of such words with underscores.\n";
    print "Output is sent to stdout.\n\n";

    print "OPTIONS:\n\n";

    print "  --help             Prints this help message.\n\n";

    print "  --version          Prints the version number of this program.\n\n";
}

# function to output the version number
sub showVersion
{
    print "compoundSval2.pl -  Version 0.1\n";
    print "Copyright (C) 2001-2002, Ted Pedersen & Satanjeev Banerjee\n";
}

# function to output "ask for help" message when the user's goofed up!
sub askHelp 
{
    print "Type compoundSval2.pl --help for help.\n";
}
